# DOMAIN LEDGER
MARKER_ATTR = "1"
MARKER_SCHEMA = "2"
MARKER_CLAIM_DEF = "3"
MARKER_REVOC_DEF = "4"
MARKER_REVOC_REG_ENTRY = "5"
MARKER_REVOC_REG_ENTRY_ACCUM = "6"
MARKER_CONTEXT = "7"
MARKER_RS_SCHEMA = "8"
LAST_SEQ_NO = "lsn"
VALUE = "val"
LAST_UPDATE_TIME = "lut"

# CONFIG LEDGER
MARKER_AUTH_RULE = "1"
